//
// TI-Memory Map Identifiers for C programs
//   Includes 68K Vectors, Low-Memory Globals, and Ports
// v1.0
// Bryan Turner
//
// These values may be used like local variables from your program, set or test them at will!
//
// This file is NOT ROM 2.1 compatable.

#ifndef _TI_RAM_
#define _TI_RAM_


//
// 68000 Chip Vectors (all calcs)
//
long TI_Reset_Stack: 0x0000;
long TI_Reset_PC: 0x0004;
long TI_BusErr_vec: 0x0008;
long TI_AddrErr_vec: 0x000C;
long TI_Illegal_vec: 0x0010;
long TI_DivZero_vec: 0x0014;
long TI_CHK_vec: 0x0018;
long TI_TRAPV_vec: 0x001C;
long TI_Priv_vec: 0x0020;
long TI_Trace_vec: 0x0024;
long TI_A_Line_vec: 0x0028;
long TI_F_Line_vec: 0x002C;
long TI_UninitInt_vec: 0x003C;
long TI_SpurInt_vec: 0x0060;

long TI_AutoVec1: 0x0064;	// Clock interrupt ~350Hz
long TI_AutoVec2: 0x0068;	// KeyDown Int ~600Hz
long TI_AutoVec3: 0x006C;	// Not Used
long TI_AutoVec4: 0x0070;	// Link Service
long TI_AutoVec5: 0x0074;	// Unknown ~18Hz
long TI_AutoVec6: 0x0078;	// ON Pressed
long TI_AutoVec7: 0x007C;	// Memory Protect Violation

long TI_TRAP_0_vec: 0x0080;	// Do some functions..
long TI_TRAP_1_vec: 0x0084;	// Change Interrupt Mask
long TI_TRAP_2_vec: 0x0088;	// Reset Calc
long TI_TRAP_3_vec: 0x008C;
long TI_TRAP_4_vec: 0x0090;	// Turn on/off calc
long TI_TRAP_5_vec: 0x0084;
long TI_TRAP_6_vec: 0x0088;
long TI_TRAP_7_vec: 0x008C;
long TI_TRAP_8_vec: 0x00A0;
long TI_TRAP_9_vec: 0x0084;	// Do some functions..
long TI_TRAP_A_vec: 0x0088;	// Self Test
long TI_TRAP_B_vec: 0x008C;
long TI_TRAP_C_vec: 0x00B0;
long TI_TRAP_D_vec: 0x0084;
long TI_TRAP_E_vec: 0x0088;
long TI_TRAP_F_vec: 0x008C;

//
// Low level globals for
// ROM 1-2 through 1-12
//

typedef struct
	{
	long InitVal;					// initial value of timer
	long CurVal;					// current value of timer; decremented every time
									// Auto-Int 5 is triggered. When it reaches zero, it
									// is reset to its initial value.
	} CountDownTimer;

typedef struct
	{
	long InitVal;					// initial value of timer
	long CurVal;					// current value of timer; decremented every time
									// Auto-Int 5 is triggered. When it reaches zero, it
									// is reset to its initial value.
	long ProcPtr;
	} CallBackTimer;

typedef struct
	{
	short head;
	short tail;
	short size;
	short used;
	short  buffer[];
	} CircularByteBuffer;
	
typedef struct
	{
	long  CurVideoBuff;
	char  WidthMinusOne;
	char  HeightMinusOne;
	short Font;
	short LineSize;
	short PenX;
	short PenY;
	char  ClipLeft;
	char  ClipTop;
	char  ClipRight;
	char  ClipBottom;
	} ScreenStructure;

long TI_BeginUserStack: 0x00120;
long TI_BeginSuperStack: 0x04120;
long TI_BeginLCDBuff: 0x04440;
char TI_Contrast: 0x05340;

char TI_BreakIndicator: 0x05342;	// (nonzero = [ON] has been pressed)

CountDownTimer TI_APD_timer: 0x05348;
CountDownTimer TI_timers[6]: 0x05348;

short TI_Timer_Expired[6]: 0x05380;	// When a timer expires, this word is set to 0001.



CallBackTimer TI_CallBackTimers[2]: 0x0538E;
	
CircularByteBuffer TI_XmitBuffer: 0x053E8;
CircularByteBuffer TI_RecvBuffer: 0x05470;

char TI_BuffersFlushed1: 0x054F8;	// set to 00 when link buffers are flushed
char TI_Reentrant: 0x054F9;			// set to FF upon entering link ISR, set to 00 upon exiting
									//     and when link buffers are flushed
char TI_BuffersFlushed2: 0x054FA;	// set to 00 when link buffers are flushed
char TI_BuffersFlushed3: 0x054FB;	// set to FF when link buffers are flushed
	  								// set to 00 when link buffers are flush2-ed
char TI_BuffersFlushed3: 0x054FC;	// set to 00 when link buffers are flush2-ed

short TI_SavedIntMask: 0x054FE;		// saved interrupt mask (see calls 4029C6 and 4029D2)

long TI_SSP: 0x05508;				// SSP (saved when calculator turns off)
long TI_OnOffMAGIC: 0x0550C;		// set to ________ when calc turns off, set to 0 when calc turns on
long TI_MemoryChecksum: 0x05510;	// checksum from 078B4 to [05536] plus from 00120 to 00FFF (saved
	  								// when calculator turns off)

char TI_MaxContrast: 0x05518;
long TI_EndOfRAM: 0x0551A;
long TI_SavedState: 0x05520;		// pointer to saved state information ($38 byte structure)

//056A0.L	: {estack_max_index}
//056A4.L	: {top_estack}
//056A8.W	: handle of estack
//056AA.L	: pointer to bottom of estack

short TI_CurBASICProg:	0x056B0;	// handle of currently running TI-BASIC program (zero if none)
long  TI_TopBASICToken:	0x056B2;	// pointer to top TI-BASIC token
long  TI_CurBASICToken:	0x056B6;	// pointer to current TI-BASIC token

//056C4.L	: pointer to string for get_lower_char functions
//056C8.B	: character result of get_lower_char functions

/*
05764.L	: menu pointer for 1:Home
05768.L	: menu handle for  1:Home
0576A.L	: menu pointer for 2:Y= Editor
0576E.L	: menu handle for  2:Y= Editor
05770.L	: menu pointer for 3:Window Editor
05774.L	: menu handle for  3:Window Editor
05776.L	: menu pointer for 4:Graph
0577A.L	: menu handle for  4:Graph
0577C.L	: menu pointer for 5:Table
05780.L	: menu handle for  5:Table
05782.L	: menu pointer for 6:Data/Matrix Editor
05786.L	: menu handle for  6:Data/Matrix Editor
05788.L	: menu pointer for 7:Program Editor
0578C.L	: menu handle for  7:Program Editor
0578E.L	: menu pointer for 8:Geometry
05792.L	: menu handle for  8:Geometry
05794.L	: menu pointer for 9:Text Editor
05798.L	: menu handle for  9:Text Editor
*/

/*
05820	: (002A) window structure
0584A.W	: ?
0584C	: (002A) window structure
05876.W	: ?
05878	: (002A) window structure for PrgmIO
*/

/*
05904.W	: name length of variable in        7:Program Editor
05906.L	: pointer after name of variable in 7:Program Editor
0590A	: $00 + folder\name of variable in  7:Program Editor

05946	: (002A) window structure for       7:Program Editor

05A28.L	: pointer after name of variable in 9:Text Editor
05A2C	: $00 + folder\name of variable in  9:Text Editor

05A68	: (002A) window structure for       9:Text Editor

05AD2	: (002A) window structure for 6:Data/Matrix Editor
*/

//05BA0.W	: {colint}
//05BA2.W	: {rowint}
	
ScreenStructure TI_ScreenStruc: 0x05C10;
	
short TI_CurCharHeight: 0x05C22;
short TI_CurCharWidth: 0x05C28;
short TI_CurTextHeight: 0x05C2A;

short TI_ScreenByteWidth: 0x05C30;

//05C44	: (0008) rect: used to store tempory bounding box)
//05C4C	: (0008) rect: used to store tempory bounding box)

long TI_CurWindowPtr: 0x05C54;

/*
05C58	: (002A) window structure used to store standard window
05CA6	: (002A) window structure
05CD0.W	: used to store a copy of 057B8
05CD2	: (0012) used to store a copy of the screen structure at 05C10
*/

short TI_CustomMenuLabelsHand: 0x05D04;
short TI_CustomMenuActionsHand: 0x05D06;

short TI_NumHandles: 0x05D2C;
long  TI_FirstFreeBytePtr: 0x05D2E;
//05D32.L	: get mem scratch var? contents depend on what was last used..
long TI_TopOfHeap: 0x05D36;
long TI_HandleTablePtr: 0x05D42;
short TI_FolderListHand: 0x05D8C;
short TI_MainFolderHand: 0x05D8E;	// (?)
short TI_MainFolderHand2: 0x05D72;	// (?)

//05DE4	: (002A) window structure for 4:Graph
//05E38	: (002A) window structure for 3:Window Editor
//05E8C	: (002A) window structure for 5:Table
//065CE	: (002A) window structure for 2:Y= Editor
//06B0E	: (002A) window structure for 8:Geometry
//07512.W	: 1=display graph number (i.e. if there are two graphs activated)
//0755E	: (002A) window structure

short TI_KeypadScancode: 0x07594;
short TI_KeyModifiers: 0x07596;

short TI_KeypadScancode2: 0x075A0;	// (same as 07594?)
short TI_KeyModifiers2: 0x075A2;	// (same as 07596?)

CircularByteBuffer TI_KeyBuffer: 0x075AA;

//075DA	: buffer for storing link packets

//075E4.W	: number of items in history
//075E6.W	: item in history currently pointed to

#define STATLINE_2ND 0x01
#define STATLINE_DIAMOND 0x02
#define STATLINE_SHIFT 0x04
#define STATLINE_HAND 0x08

#define STATLINE_2ND_HAND 0x20

// Angle: 0=RAD  1=DEG
#define STATLINE_ANGLE 0x40

// Exact/Approx:  0/3=AUTO 1=EXACT 2=APPROX
#define STATLINE_EXACT_APPROX 0x0180

// CurGraph: (if split screen) 0 = graph#1   1 = graph#2
#define STATLINE_CURGRAPH 0x0200

// Graph Mode: 1=FUNC 2=PARA 3=POL 4=SEQ 5=3D
#define STATLINE_GRAPHMODE 0x1C00

// Activity: 0=idle 1=busy 2=paused
#define STATLINE_ACTIVITY 0x6000
#define STATLINE_UPDATE_NOW 0x8000

long TI_StatusLineFlags: 0x0761C;
char TI_BeginHeap: 0x078B4;


/*
:RW [600000]
    <.....2..|........> = something to do with keyboard
    <..5.....|........> = bits <....0> of contrast
    <........|.......0> = clear: interleave RAM (allows use of 256K of RAM)
    <........|.....2..> = set: generate Auto-Int 7 when memory below [000120]
			: is written
*/

#define MEMORY_PROTECT 0x04

short TI_Port00s: 0x600000;
char TI_Port00b: 0x600000;
char TI_Port01b: 0x600001;

/*
:WO [600002]: Tleilax, See Hardware Grayscale at $600012 below...
    <........|......1.> = Switches to 4 graylevels if you wrote $10 at $600012
*/

#define HWGRAY_ON 0x000A

short TI_Port02s: 0x600002;
char TI_Port02b: 0x600002;
char TI_Port03b: 0x600003;

/*
:WO [600004]
    <........|....3...> = set: 000000..1FFFFF mapped to 200000..3FFFFF
*/

short TI_Port04s: 0x600004;
char TI_Port04b: 0x600004;
char TI_Port05b: 0x600005;

/*
:WO [600006]
    <........|........> =

:WO [600008]
    <........|........> =

:WO [60000A]
    <........|........> =
*/
short TI_Port06s: 0x600006;
char TI_Port06b: 0x600006;
char TI_Port07b: 0x600007;

short TI_Port08s: 0x600008;
char TI_Port08b: 0x600008;
char TI_Port09b: 0x600009;

short TI_Port0As: 0x60000A;
char TI_Port0Ab: 0x60000A;
char TI_Port0Bb: 0x60000B;

/*
:RW [60000C]
    <765.3210|........> = (write) link status
    <........|7...321.> = (read) link status
    <........|..5.....> = (read) set: one byte receive buffer has a byte
    <........|.6......> = (read) set: one byte transmit buffer is empty

To control the Link Port lines individually: [Bryan Turner]
Move $40 to $60000C.b
Set Bits 0 and 1 of $60000E.b to control the output of Red and White wires
Test Bits 2 and 3 of $60000E.b to examine input on Red and White wires
When finished move $8D to $60000C.b
*/

#define LINKCNTL_ON 0x40
#define LINKCNTL_OFF 0x8D

short TI_Port0Cs: 0x60000C;
char TI_Port0Cb: 0x60000C;
char TI_Port0Db: 0x60000D;

/*
:RW [60000E]
    <.......0|........> = ? link control Red
    <......1.|........> = ? link control White
    <.....2..|........> = ? red output
    <....3...|........> = ? white output
    <........|76543210> = read a byte from the receive buffer (1 byte buffer)
    <........|76543210> = write a byte to the transmit buffer (1 byte buffer)
*/

#define RED_CNTL 0x01
#define WHITE_CNTL 0x02

#define RED_STATUS 0x04
#define WHITE_STATUS 0x08

short TI_Port0Es: 0x60000E;
char TI_Port_LinkControl: 0x60000E;
char TI_Port_LinkBuffer: 0x60000F;

//:WO [600010]
//    <76543210|76543210> = Address of LCD memory divided by 8

short TI_Port_LCDAddr: 0x600010;

/*
:WO [600012]
    <..543210|........> = LCD horizontal frequency (?maybe?)
			: The TI-92's ROM writes $31 to this port during
			: initialization.
			
	 <........|76543210> = $100 - number of LCD scanlines
			: If the number of scanlines is smaller than $80 (the
			: actual height of the LCD) the display will be
			: duplicated in the lower half. Decreasing the height
			: darkens the LCD; increasing the height lightens the
			: LCD. The TI-92's ROM writes $80 to this port during
			: initialization.

Hardware Grayscale:  Tleilax, edited and updated by Bryan Turner
Create 3 bitplanes consecutively in memory.
Store bitplane address/8 into $600010
Move $11 to $600012
Move $0A to $600003

Clean up by moving #31 to $600012
Interrupts do NOT have to be disabled.

($600012):
  $10 -> 32 bytes lines + 4 graylevels (3 bitplanes following each other)
  
  Bryan Turner: Lower 4 bits cause line size to change:
     $x0 = 32 bytes/line
     $x1 = 30 bytes/line
     $x2 = 28 bytes/line
     etc..

*/

char TI_Port_ScreenCntl: 0x600012;
char TI_Port_ScanLines: 0x600013;


/*:RW [600014]
    <........|........> =

($600015): Tleilax
  $00 -> Triggers Int 3
  $01 -> turns the screen off and freezes
*/

short TI_Port14s: 0x600014;
char TI_Port14b: 0x600014;
char TI_Port15b: 0x600015;

/*
:RW [600016]
    <........|76543210> = Programmable rate generator. Set the timer's init-
			: ial value by writing it to this port. The timer is
			: incremented every 6250 clock cycles, unless it has
			: a value of zero, in which case it is reset to its
			: initial value. The LCD is refreshed every 16th time
			: this timer is incremented.
			:
			: See also Auto-Ints 1 and 5 in TRAPS.TXT.
*/

short TI_Port16s: 0x600016;
char TI_Port16b: 0x600016;
char TI_Port_RateGen: 0x600017;

/*
:RW [600018]: SEE KEYBOARD MATRIX INFO AT BOTTOM
    <......10:76543210> = keyboard row mask; setting a bit masks the
			: corresponding row of the keyboard from being
			: read by [60001B].
*/

short TI_Port_KbdRowMask: 0x600018;

/*
:RO [60001A] SEE KEYBOARD MATRIX INFO AT BOTTOM
    <......1.|........> = ON key status (0=down, 1=up)
    <........|76543210> = keyboard column mask; if a bit is clear, one or
			: more keys in the corresponding column are being
			: held down. Keys in rows masked by [600018] are
			: ignored.
*/

short TI_Port_KeysDown: 0x60001A;

/*
:WO [60001C]
    <...43210|........> = Something to do with LCD
			: The TI-92's ROM writes $21 to this port during
			: initialization, and $FF when the calculator is
			: turned off.
	
	<........|....3210> = bits <4321.> of contrast
	
($60001c): Tleilax
  $f0 -> magnify the screen vertically (strange)
  $30 -> idem
  bit 3 seems to horizontally unmagnify the screen
  $3c -> a line at the bottom of the screen
  $3d -> idem
  $3e -> idem at the top
  $3f -> idem a bit lower
*/

short TI_Port1Cs: 0x60001C;
char TI_Port1Cb: 0x60001C;
char TI_Port1Db: 0x60001D;

//:WO [60001E]
//   <........|........> =

short TI_Port1Es: 0x60001E;
char TI_Port1Eb: 0x60001E;
char TI_Port1Fb: 0x60001F;

/*
-----------------------------------------------------------------------------
THE KEYBOARD MATRIX
-----------------------------------------------------------------------------

As was hinted in the I/O ports section, the keyboard is accessed internally
as a matrix with 10 rows and 8 columns. This matrix can be read by writing
[600018], pausing to allow the I/O to recover, then reading [60001B].

Row     +-------+-------+-------+-------+-------+-------+-------+-------+
 V  Col>| Bit 7 | Bit 6 | Bit 5 | Bit 4 | Bit 3 | Bit 2 | Bit 1 | Bit 0 |
+-------+-------+-------+-------+-------+-------+-------+-------+-------+
| Bit 0 |  down | right |   up  |  left |  hand | shift |diamond|  2nd  |
| Bit 1 |   3   |   2   |   1   |   F8  |   W   |   S   |   Z   | unused|
| Bit 2 |   6   |   5   |   4   |   F3  |   E   |   D   |   X   | unused|
| Bit 3 |   9   |   8   |   7   |   F7  |   R   |   F   |   C   |  STO  |
| Bit 4 |   ,   |   )   |   (   |   F2  |   T   |   G   |   V   | space |
| Bit 5 |  TAN  |  COS  |  SIN  |   F6  |   Y   |   H   |   B   |   /   |
| Bit 6 |   P   | ENTER2|   LN  |   F1  |   U   |   J   |   N   |   ^   |
| Bit 7 |   *   |  APPS | CLEAR |   F5  |   I   |   K   |   M   |   =   |
| Bit 8 | unused|  ESC  |  MODE |   +   |   O   |   L   | theta |backspc|
| Bit 9 |  (-)  |   .   |   0   |   F4  |   Q   |   A   | ENTER1|   -   |
+-------+-------+-------+-------+-------+-------+-------+-------+-------+

Note: ENTER1 is on the alphabetic _and_ numeric keypads.
      ENTER2 is next to the cursor pad.

Because of the way the TI-92's keyboard is wired, if you hold down three
keys that form the corners of a rectangle, the TI-92 will think you are
also holding down the key at the fourth corner.

The [ON] key is special, and is not part of the matrix.
*/



#endif
