//
// Xinu Header (Process Version)
//
// Xinu for the TI-92
//
// By Bryan Turner

#define	SYSCALL		long
#define LIBFUNC		asm
#define XinuLib(x) { moveq %(x),d0; trap %14; }

typedef struct { void; } *SEM_ID;
typedef struct { void; } *PROC_ID;
typedef long WORD;

// Library Functions
LIBFUNC short Xinu_LoadLibrary( char *LibName, long *LibData )
	XinuLib(0)
LIBFUNC short Xinu_UnloadLibrary( long *LibData )
	XinuLib(1)

// Process Functions
LIBFUNC PROC_ID Xinu_CreateProc( void *procaddr, short ssize, short priority, short nargs, long args)
	XinuLib(2)
LIBFUNC SYSCALL Xinu_KillProc( PROC_ID theProcess )
	XinuLib(3)
LIBFUNC SYSCALL Xinu_Resume(PROC_ID theProcess)
	XinuLib(4)
LIBFUNC SYSCALL	Xinu_Suspend(PROC_ID theProcess)
	XinuLib(5)
LIBFUNC SYSCALL	Xinu_GetSetState( PROC_ID theProcess, long data, short what )
	XinuLib(6)
LIBFUNC SYSCALL	Xinu_Sleep( short n )
	XinuLib(7)

#define Xinu_ChangePrio( a, b ) Xinu_GetSetState( (a), (b), 0)
#define Xinu_GetPID() Xinu_GetSetState( (PROC_ID)-1, 0, 0 )
#define Xinu_GetPrio( a ) Xinu_GetSetState( (a), 0, 0 )
#define Xinu_SetNOK( nok, proc ) Xinu_GetSetState( (proc), (nok), 1 )


// Message Functions
LIBFUNC SYSCALL	Xinu_Receive( short maxwait )
	XinuLib(8)
LIBFUNC SYSCALL	Xinu_Send( PROC_ID theProcess, WORD msg )
	XinuLib(9)

// Semaphore Functions
LIBFUNC SEM_ID Xinu_SemCreate( short count )
	XinuLib(10)
LIBFUNC SYSCALL Xinu_SemDelete( SEM_ID theSemaphore )
	XinuLib(11)
LIBFUNC SYSCALL Xinu_MultiSignal( SEM_ID theSemaphore, short count )
	XinuLib(12)
LIBFUNC SYSCALL	Xinu_Wait( SEM_ID theSemaphore )
	XinuLib(13)
LIBFUNC SYSCALL Xinu_GetSetSemCount( SEM_ID theSemaphore, short count )
	XinuLib(14)

#define Xinu_Signal( a ) Xinu_MultiSignal( (a), 1 )
#define Xinu_SemCount( a ) Xinu_GetSetSemCount( (a), -1 )
#define Xinu_SemReset( a, b ) Xinu_GetSetSemCount( (a), (b) )

// Misc. Functions
LIBFUNC SYSCALL	Xinu_FreeMem(void *block, unsigned int size)
	XinuLib(15)
LIBFUNC WORD *Xinu_GetMem(unsigned int nbytes)
	XinuLib(16)

// Misc. Functions
LIBFUNC void Xinu_kprintf( char *fmt, ...)
	XinuLib(17)

LIBFUNC short Xinu_GetKey()
	XinuLib(18)

//
