//* clkint.s -  clkint


#include <Globals.h>
#include "TIOS.h"

//*------------------------------------------------------------------------
//* clkint  --  real-time clock interrupt service routine 
//*------------------------------------------------------------------------

#undef defclk
#undef clkdiff
#undef areProcsAsleep
#undef sltop

void AsmClockInit();
INTPROC	wakeup();

asm int clkint()
{
/*
// BT: For using the clock int on a faster interrupt (Auto-Int 1),
//     the interrupt rate will have to be divided, this code does this.
//     (divides the rate by 4 to be specific)
//     Right now the clock is on Auto-Int 5, so the rate does not need dividing.

	move.l	a0,-(a7)
	lea		counter,a0
	addq.b	%1,(a0)						// Add one to the counter
	andi.b	%3,(a0)						// Flip counter when it's too high
	movea.l	(a7)+,a0
	beq		clockInt					// When it flips, call the interrupt
	bra		oldClockInt					// Otherwise, skip it and call the old one..
counter:
	dc.w	0							// Counts up then resets
										// Divides the clock tick rate
*/
	
	//
	// The meat of the clock interrupt...
	//
clockInt:

	//
	// BT: I believe that without these lines, the system will be unstable,
	//     however I have not been able to cause the system to crash with
	//     these lines removed.  Therefore, since supervisor programs are often
	//     interrupted and the clock must receive as many ticks as possible, I've
	//     left them commented out.
	//
	//btst	%5,(a7)			// Test supervisor bit of process we preempted
	//bne		oldClockInt		// We interrupted a supervisor-mode program!
								// Exit immediately, do not interrupt!
	
	move.w	%0x2700,sr					// disable ALL interrupts
	movem.l	d0-d2/a0-a1/a4,-(a7)		// Save registers (supervisor stack)
	
	lea		actualGlobals,a4			// Load our globals

	// Sleep Queue Processing...
	tst.b	xinuGlobs.areProcsAsleep	// Is sleep queue empty?
	beq     cl_PreEmpt					//  Yes -> skip down
	
	move.l	xinuGlobs.sltop,a0
	subq.w	%1,(a0)						//  No -> decrement delta key
	bpl     cl_PreEmpt					//        of first process
	
	jsr		wakeup						// Wakeup procs, don't reshedule though..
	bra		dopreempt					// After waking, always preempt
	
cl_PreEmpt:
	subq.w	%1,xinuGlobs.preempt		// Decrement preemption counter
	bne		clret						// Not zero yet..
	
	// Preempt the current process!
dopreempt:

	// --- DEBUG CODE:
	//     Flash a little graphic to show we're working...
	lea		xinuGraphic,a0
	lea		0x4440+(128-6)*30+(30-6),a1	// --- NOT ROM 2.1 Compatable
	moveq	%5,d0
xgraph:
	move.w	(a0)+,d1
	eor.w	d1,(a1)
	add.w	%30,a1
	dbra	d0,xgraph
	

	move.w	%QUANTUM,xinuGlobs.preempt	// Reset preemption counter

//
// Super Stack:
//  0: D0-D2/A0-A1/A4
// 24: SR
// 26: PC
//

	//
	// BT: resched must be called from user mode, but we must also restore the
	//     preempted SR and PC.  So, save them on the user stack, and we'll use
	//     a special trap 13 call to replace them after rescheduling.
	//
	move	usp,a0						// Load user stack
	move.w	24(a7),-(a0)				// Store process SR (like an argument to a function)
	move.l	26(a7),-(a0)				// Store process return addr (as if we called some function)
	move	a0,usp						// Save new user stack addr
	
	lea		doresched,a1				// Load proc to call (this is the function we're calling)
	move.l	a1,26(a7)					// Edit proc to return to (on super stack)
	move.w	%0x0700,24(a7)				// Edit SR to turn off all interrupts (on super stack)
	
clret:
	movem.l	(a7)+,d0-d2/a0-a1/a4		// Restore registers (from super stack)
oldClockInt:
	jmp		0x10000001


doresched:
	//
	// This function adds some overhead to allow it to call reshced
	//    from user mode.  This is used to keep the stacks aligned
	//    no matter what mode the CPU was in when it interrupted the
	//    process to reschedule another.  Usually resched is called
	//    from a system proc which adds this overhead manually.
	//
	
	movem.l	d0-d2/a0-a1/a4,-(a7)	// Save registers (user stack)
	
	lea		actualGlobals,a4		// Load Xinu Globals
	jsr		resched					// Call resched...
	
	movem.l	(a7)+,d0-d2/a0-a1/a4	// Restore registers (user stack)
	trap	%13						// Won't return to us (returns to preempted proc)
	
trap13proc:
//
// Super Stack:
//  0: A0
//	4: SR
//	6: PC
//
	movem.l	a0,-(a7)				// Save register (super stack)
	move	usp,a0					// Load user stack
	move.l	(a0)+,6(a7)				// Edit return addr (on super stack)
	move.w	(a0)+,4(a7)				// Edit return SR
	move	a0,usp					// Save new user stack
	move.l	(a7)+,a0				// Restore regs (super stack)
	rte								// return to preempted proc & restore SR
	
	
//
// Clock Initialization procedure
//
	entry	AsmClockInit
	
	lea		oldClockInt,a0
	move.l	0x74,2(a0)					// Store old clock interrupt
	lea		clkint,a0
	move.l	a0,0x74						// Install new clock interrupt
	lea		trap13proc,a0
	move.l	a0,0xB4						// Install trap 13 processor
	rts
	

xinuGraphic:
	dc.w	0xFFFF,0xAAD5,0xAA55,0xDA15,0xAA95,0xAAD1	// --- Graphic 'XINU' displayed on context switch
}

//
// BT: This interrupt is installed to activate and deactivate the GetKey function
//

asm AsmKeyInit();

asm void GetKeyInt()
{
	cmp.b	#0x21,TI_ST_flags+3	// If status line says 2nd-Hand-2nd has been pressed...
	bne.s	doNorm

	movem.l	d0/a0/a4,-(a7)
	
	move.b	#0,TI_ST_flags+3	// Clear status info
	lea		branchStore,a0		// Load toggle code
	lea		doNorm,a4
	
	move.w	(a0),d0				// Switch branch & nop to turn Xinu Focus on or off
	move.w	(a4),(a0)			//   (self-modifying code)
	move.w	d0,(a4)
	
	moveq	#-1,d0				// -1 Means interrupt is OFF
	cmpi.w	#0x4e71,d0			// If we just installed interrupt though..
	bne		doMark
	
	moveq	#0,d0				// Mark interrupt as ON
doMark:
	lea		actualGlobals,a4
	move.w	d0,xinuGlobs.curKey	// Mark key interception as ON/OFF
	
	movem.l	(a7)+,d0/a0/a4
	
doNorm:
	bra.s	doInt			// This is replaced with a NOP when int is installed
	
	pea		doGetKey		// Push our address so after keyboard interrupt we will be called.
	move.w	#0x2700,-(a7)	// Build interrupt-disabled stack frame
doInt:
	jmp		0x10000001		// Jump to old interrupt
	
	
doGetKey:
	//
	// If we are installed, the interrupt returns here with interrupts disabled
	// Read key from buffer and remove...
	//
	
	tst.w	(TI_kb_vars+0x1C)		// If there is data there...
	beq		doInt
	
	movem.l	d0/a4,-(a7)
	moveq	#0,d0
	move.w	d0,TI_kb_vars+2			// Clear some other keyboard variables..
	move.w	d0,TI_kb_vars+6
	move.w	d0,TI_kb_vars+0x1C		// Tell TI-OS there is nothing (steal it!)
	move.w	(TI_kb_vars+0x1E),d0	// Get the keypress
	lea		actualGlobals,a4		// Load the Globals
	move.w	d0,xinuGlobs.curKey		// Store the data into the current key slot
	movem.l	(a7)+,d0/a4
	
	rte								// This was a tail patch, so now return to the interrupted process

	// Initialize the Keyboard interrupt stuff...
	entry AsmKeyInit
	lea		doInt,a0
	move.l	0x64,2(a0)					// Store old keyboard interrupt
	lea		GetKeyInt,a0
	move.l	a0,0x64						// Install new keyboard interrupt
	rts

branchStore:
	nop									// Place to store branch while int is installed.

}

short GetKey()
{
	short ch;
	short sr;
	long a4;
	
	a4 = SaveA4AndLoadGlobals();
	sr = disable();
	
	ch = xinuGlobs.curKey;
	if ( xinuGlobs.curKey > 0 )
		xinuGlobs.curKey = 0;
	
		
	RestoreA4AndSR(a4,sr);
	return ch;
}
