//
// Library Interface Module
//
// Bryan Turner

#include <Xinu.h>

short LoadLibrary( char *LibName, char **LibData );
short UnloadLibrary( long *LibData );


//
// Trap Mechanism to acquire supervisor mode from user mode processes
//
asm	void Trap14Exception()
{	
	// D0 contains function number to call
	// Multiply by 4, then jump to it!
	
	move.w	#0x2700,sr			// Disable interrupts (would corrupt supervisor stack otherwise)

	//
	// Stack is not correct for call to system functions.
	// Stack currently contains SR and PC, then arguments.
	// First Pop off the SR, the push on the library function address,
	//    then put the SR back on top.
	// Now when we return the Library func will be entered as if it was called directly.
	//
	lsl.w	#2,d0				// Multiply D0 by 4
	
	move.w	(a7)+,d1			// Pop SR off stack
	addq	#4,a7				// Throw out old return addr
	
	lea		XinuJumpTable,a0
	pea		0(a0,d0.w)			// Push system function addr on stack
	move.w	d1,-(a7)			// Push SR (create exception frame)
	rte							// Return to system function, which will return to process
	
XinuJumpTable:					// WARNING: Do NOT re-arrange system calls, only add them to the end of the list!!!
// Library Functions
	jmp		LoadLibrary			// System Call #0
	jmp		UnloadLibrary

// Process Functions
	jmp		create				// System Call #2
	jmp		kill
	jmp		resume
	jmp		suspend
	jmp		GetSetState
	jmp		sleep10
	
// Message Functions
	jmp		receive				// System Call #8
	jmp		send

// Semaphore Functions
	jmp		screate				// System Call #10
	jmp		sdelete
	jmp		signaln
	jmp		wait
	jmp		GetSetSemCount

// Memory Functions
	jmp		freemem				// System Call #15
	jmp		getmem

// Misc. Functions
	jmp		kprintf				// System Call #17
	jmp		GetKey
	
// Add New System Calls Below This Line
// ------------------------------------


}



