/* create.c - create, newpid */

#include <Globals.h>

extern void actualGlobals();
extern PROC_ID newpid();

/*------------------------------------------------------------------------
 *  create  -  create a process to start running a procedure
 *------------------------------------------------------------------------
 */


asm void StartProc()
{
	moveq	%0,d0
	trap	%1				// Enable interrupts
	rts
}

PROC_ID create( void *procaddr, short ssize, short priority, short nargs, long args)
{
	short sr; 
	long a4;
	Process	*theProc;
	int	i;
	WORD	*a;				// points to list of args	
	WORD	*saddr;			// stack address		

	a4 = SaveA4AndLoadGlobals();
	sr = disable();
	
	ssize = (int) roundmb(ssize);
	
	if ( ssize < MINSTK )	// BT: No longer an error
		ssize = MINSTK;
	if ( priority < 1 )		// BT: No longer an error
		priority = 1;
	
	if ( ((saddr = getstk(ssize)) == (WORD *)SYSERR ) ||
		(theProc=(Process *)newpid()) == (Process *)SYSERR ||
		isodd(procaddr) )
		{
		if ( saddr != (WORD *)SYSERR )	// BT: was not deallocating on error!
			freestk( saddr, ssize );
		RestoreA4AndSR(a4,sr);
		return (PROC_ID)SYSERR;
		}
	

	theProc->qhand.flags = PROC_FLAG;
	theProc->pstate = PRSUSP;	// BT: Begins in suspended state
	theProc->pprio = priority;
	theProc->pbase = (WORD) saddr;
	theProc->pstklen = ssize;
	theProc->psem = 0;
	if ((theProc->msgQueue = NewQueue()) == (Q_ID)SYSERR )
		{
		// Error! --- BT: Do something appropriate...
		}
	//theProc->plimit = theProc->pbase - ssize + sizeof (WORD);
	
	//
	// BT
	// Build new process's stack:
	//   0: D0-D7/A0-A6
	//  60: RTEProc address
	//  64: Status Register
	//  66: Process' address (return address from context switch)
	//  6A: userret()  (proc to call when this process terminates)
	//  6E: # Args
	//  78: Args...
	//  xx: MAGIC (bottom of stack)
	//
	*(saddr--) = MAGIC;					// Bottom of stack
	
	a = (&args) + (nargs-1);			// point to last argument
	for ( i=0; i<nargs ; i++)			// machine dependent; copy args
		*(saddr--) = *(a--);
	*(saddr--) = nargs;
	*(saddr--) = (WORD)INITRET;			// Push on return address : userret()
	*(saddr--) = (WORD)procaddr;		// Address of process's main function
	
	// --- BT: Testing, enable interrupts first thing when procs start
	*(saddr--) = (WORD)StartProc;
	
	for (i=0 ; i < 15 ; i++)			// Registers to be restored by ctxsw()
		*(saddr--) = INITREG;
	
	saddr++;							// Decremented once too far..
	theProc->pcurstack = (WORD)saddr;	// Store stack pointer

	theProc->pnxtkin = (PROC_ID)xinuGlobs.readyQ; // BT: a Non-process ID

	RestoreA4AndSR(a4,sr);
	return (PROC_ID)theProc;
}

/*------------------------------------------------------------------------
 * newpid  --  obtain a new (free) process id
 *------------------------------------------------------------------------
 */
#undef nextproc

PROC_ID newpid()
{
	short	i,pid;
	Process *theProc;
	
	for (i=0 ; i<NPROC ; i++) {	// check all NPROC slots
		if ( (pid=xinuGlobs.nextproc--) <= 0)
			xinuGlobs.nextproc = NPROC-1;
		
		theProc = &(xinuGlobs.theProcs[pid]);
		if (theProc->pstate == PRFREE)
			return (PROC_ID)theProc;
	}
	return (PROC_ID)SYSERR;
}




